/* Auspice functions.
 *
 * Auspice Services is copyright (c) 2000-2001 In Mean.
 *     E-mail: <inmean@pleaselook.com>
 *
 * This program is free but copyrighted software; see the file COPYING for
 * details.
 */

#include "../inc/services.h"
#include "../inc/timeout.h"
#include "bs-help.c"

int servbot;
static int nhostline = 0;
static int hostline_size = 0;
static struct s_hline {
	char *nick;
	char *host;
	char who[NICKMAX];
	long reserved[4];
} *s_hlines = NULL;

char *botname[64];
char *botmaster[64];

static void do_agenthelp(const char *source);
static void do_agentset(const char *source);

int del_hostline(const char *nick);
void load_hline_db(void);
void save_hline_db(void);
void do_hostline(const char *nick);
void agent(const char *source, char *buf);
void servicesX(const char *source, char *buf);
void servicesW(const char *source, char *buf);
void do_selfdel(const char *source);

/*************************************************************************/
void load_hline_db(void)
{
    FILE *f;
    int i;

    if (!(f = open_db(s_ServicesW, HLINE_DB, "r")))
        return;

    switch (i = get_file_version(f, HLINE_DB)) {
      case 5:
        nhostline = fgetc(f) * 256 + fgetc(f);
        if (nhostline < 8)
            hostline_size = 16;
        else
            hostline_size = 2*nhostline;
        s_hlines = smalloc(sizeof(*s_hlines) * hostline_size);
        if (!nhostline) {
            fclose(f);
            return;
        }
        if (nhostline != fread(s_hlines, sizeof(*s_hlines), nhostline, f))
            fatal_perror("Read error on %s", HLINE_DB);
        for (i = 0; i < nhostline; i++) {
            s_hlines[i].nick = read_string(f, HLINE_DB);
            s_hlines[i].host = read_string(f, HLINE_DB);
        }
        break;
      default:
        fatal("Unsupported version (%d) on %s", i, HLINE_DB);
    } /* switch (version) */

    close_db(f, HLINE_DB);
}
/***********************************************************/
void load_bot_db(void)
{
    FILE *f;
    char botdata[128];
    int i;
    *botdata = 0;
    servbot = 1;
    strcat(botdata, "bot.db");
    if (f = fopen(botdata, "r")) {
    while(fgets(botdata, sizeof(botdata), f)) {
	 if (!(botname[servbot])) {
		botname[servbot] = sstrdup(strtok(botdata, "\n"));
	 } else {
		botmaster[servbot] = sstrdup(strtok(botdata, "\n"));
		servbot++;
	}
    }
    fclose(f);
}
    servbot--;
}
/***********************************************************/
void connectbot(void)
{
    int i;
    if (servbot>0) {
    for (i=1; i<=servbot; i++) {
    send_cmd (NULL, "NICK %s 1 %lu %s %s %s %lu :%s",
    botname[i], time(NULL), "ServerBot", services_host, server_name, time(NULL), "Services Bot");
    send_cmd(botname[i], "MODE %s +dqSB", botname[i]);
    send_cmd(botname[i], "JOIN %s", snoopchan);
    send_cmd(botname[i],"SWHOIS %s :is a Services Bot",botname[i]);
    } }
}
/***********************************************************/
void save_hline_db(void)
{
    FILE *f;
    int i, count = 0;

    f = open_db(s_ServicesW, HLINE_DB, "w");
    fputc(nhostline/256, f);
    fputc(nhostline & 255, f);
    if (fwrite(s_hlines, sizeof(*s_hlines), nhostline, f) != nhostline)
        fatal_perror("Write error on %s", HLINE_DB);
    for (i = 0; i < nhostline; i++) {
        write_string(s_hlines[i].nick, f, HLINE_DB);
        write_string(s_hlines[i].host, f, HLINE_DB);
    }
    close_db(f, HLINE_DB);

}
/**********************************************************/
void save_bot_db(void)
{
    FILE *f;
    int i;
    f = fopen("bot.db", "w");
    for	(i=1; i<=servbot; i++) {
        fprintf(f, "%s\n", botname[i]);
        fprintf(f, "%s\n", botmaster[i]);
    }
    fclose(f);
}
/**********************************************************/
int is_hostline(const char *nick)
{
    NickInfo *ni;
    int i;

    for (i = 0; i < nhostline; i++) {
        if (stricmp(s_hlines[i].nick,nick) == 0) {
            return 1;
        }
    }
    return 0;
}
/*********************************************************/
int is_botowner(const char *nick, const char *buf)
{
    int i;
    for (i = 1; i <= servbot; i++) {
        if ((stricmp(botname[i],buf) == 0) && (stricmp(botmaster[i],nick)==0)){
            return 1;
        }
    }
    return 0;
}
/*********************************************************/
void do_hostline(const char *nick)
{
    NickInfo *ni;
    int i;
    for (i = 0; i < nhostline; i++) {
        if (stricmp(s_hlines[i].nick,nick) == 0) {
	    send_cmd(s_Agent,"CHGHOST %s %s",nick,s_hlines[i].host);
	    return;
        }
    }
    return;
}
/**********************************************************/
int del_hostline(const char *nick)
{
    int i;

    for (i = 0; i < nhostline && stricmp(s_hlines[i].nick, nick) != 0; i++);
    if (i < nhostline) {
        free(s_hlines[i].nick);
        free(s_hlines[i].host);
        nhostline--;
        if (i < nhostline)
        bcopy(s_hlines+i+1, s_hlines+i, sizeof(*s_hlines) * (nhostline-i));
        return 1;
    } else {
        return 0;
    }
}
/***********************************************************/
void agent(const char *source, char *buf)
{
    char *cmd;
    char *s, *chan, *nick, *reason, *otherc;
    int i;
    User *u = finduser(source);

    cmd = strtok(buf, " ");

   if (!is_services_admin(source)) {
       	notice(s_Agent, source, "Under development.");
       	return;
   }

    if (!cmd) {
        return;
    } else if (stricmp(cmd, "HELP") == 0) {
        do_agenthelp(source);
    } else if (stricmp(cmd, "OP") == 0) {
	chan = strtok(NULL, " ");
	nick = strtok(NULL, "");
	if (!chan) {
		notice(s_Agent, source, "Syntax: OP \2Channel\2 \2Nick\2");
		return;
	}
	if (!nick) {
                if (!is_chanop(source, chan))
	        change_cmode (s_Agent, chan, "+o", source);
		return;
	}
	else {
                if (!is_chanop(source, chan))
	        change_cmode (s_Agent, chan, "+o", nick);
		return;
	}

    } else if (stricmp(cmd, "NICK") == 0) {
	chan = strtok(NULL, " ");
	nick = strtok(NULL, "");
	if (!chan) {
		notice(s_Agent, source, "Syntax: NICK \2nick\2 \2nick\2");
		return;
	}
	if (!nick) {
		notice(s_Agent, source, "Syntax: NICK \2nick\2 \2nick\2");
		return;
	}
	else {
	        send_cmd (s_Agent, "SVSNICK %s %s 0", chan, nick);
		return;
	}

    } else if (stricmp(cmd, "\1PING") == 0) {
        if (!(s = strtok(NULL, "")))
            s = "\1";
        notice(s_OperServ, source, "\1PING %s", s);
    } else if (stricmp(cmd, "LISTCHANS") == 0) {
	do_aglist(source);
    } else if (stricmp(cmd, "SET") == 0) {
	do_agentset(source);
    } else if (stricmp(cmd, "INFO") == 0) {
        notice(s_Agent, source, "Services Settings Info:");
        notice(s_Agent, source, " ");
        notice(s_Agent, source, "X set mode +oa in Mark/Hold Channel: \2%s\2",(CHANX == 1)?"Enable":"Disable");
        notice(s_Agent, source, "WebTV support, all notice message show in PRIVMSG: \2%s\2", (SPRIV == 1)?"Enable":"Disable");
        notice(s_Agent, source, "Services Database auto update (Risking): \2%s\2", (AUTOUP == 1)?"Enable":"Disable");
        notice(s_Agent, source, "Hiding Services next update info: \2%s\2", (HIDEUME == 1)?"Enable":"Disable");
        notice(s_Agent, source, "Allow user do NS ListChans: \2%s\2", (ALLOWCLIST == 1)?"Enable":"Disable");
        notice(s_Agent, source, "\2%s\2 force user join channel: \2%s\2 on connect.", (AUTOJOINC == 1)?"Enable":"Disbale", ACHANS);
        notice(s_Agent, source, "\2%s\2 force oper join channel: \2%s\2 on oper up.", (OPERJOIN == 1)?"Enable":"Disable", AOCHANS);
        notice(s_Agent, source, "\2%s\2 adding access mask in channel access.", (NS_DACCESS == 0)?"Enable":"Disbale");
        notice(s_Agent, source, "\2%s\2 user set channel JOIN Flag.", (CJOIN == 1)?"Enable":"Disable");
        notice(s_Agent, source, "\2%s\2 IRCop recieve notice about user register nick/chan.", (REGNOTICE == 1)?"Enable":"Disable");
        notice(s_Agent, source, ":End of Services Settings Info");

    } else if (stricmp(cmd, "JOIN") == 0) {
	if (!is_services_root(source)) {
       		notice(s_Agent, source, "Permision Denied.");
	       	return;
	}
	chan = strtok(NULL, " ");
	if (!chan) {
		notice(s_Agent, source, "Syntax: JOIN \2Channel\2");
	} else {
	        send_cmd (s_Agent, "JOIN %s", chan);
        	send_cmd (s_Agent, "MODE %s +a %s", chan, s_Agent);
	}

    } else if (stricmp(cmd, "PART") == 0) {
	if (!is_services_root(source)) {
       		notice(s_Agent, source, "Permision Denied.");
		return;
	}
	chan = strtok(NULL, " ");
	if (!chan) {
		notice(s_Agent, source, "Syntax: PART \2Channel\2");
	} else {
	        send_cmd (s_Agent, "PART %s", chan);
	}

    } else if (stricmp(cmd, "KICK") == 0) {
	if (!is_services_root(source)) {
       		notice(s_Agent, source, "Permision Denied.");
		return;
	}
	chan = strtok(NULL, " ");
	nick = strtok(NULL, " ");
	reason = strtok(NULL, "");
	if ((!chan) || (!nick) || (!reason)) {
		notice(s_Agent, source, "Syntax: KICK \2Channel\2 \2nick\2 \2reason\2");
	} else {
	    send_cmd(s_Agent, "KICK %s %s :%s", chan, nick, reason);
	}

    } else if (stricmp(cmd, "EXIT") == 0) {
	if (!is_services_root(source)) {
       		notice(s_Agent, source, "Permision Denied.");
		return;
	}
	nick = strtok(NULL, " ");
	reason = strtok(NULL, "");
	if ((!nick) || (!reason)) {
		notice(s_Agent, source, "Syntax: EXIT \2nick\2 \2reason\2");
	} else {
	    send_cmd(s_Agent, "SVSKILL %s :%s", nick, reason);
	}

    } else {
        notice(s_Agent, source,
                "Unrecognized command \2%s\2.", cmd);
    }
}
/*****************************************************************/
static void do_agenthelp(const char *source)
{
    const char *cmd = strtok(NULL, " ");
    if (!cmd) {
        notice_list(s_Agent, source, agent_help);
    } else if (stricmp(cmd, "SET") == 0) {
        notice_list(s_Agent, source, agent_helpset);
    } else {
        notice(s_Agent, source,	"No help available for command \2%s\2.", cmd);
    notice(s_Agent, source, "\2END OF HELP\2");
    }
}
/******************************************************************/
static void do_agentset(const char *source)
{
    const char *cmd = strtok(NULL, " ");
    char *s, *chan, *nick, *reason, *otherc, *reservc;
    int i;
    User *u = finduser(source);

    if (!cmd) {
        notice_list(s_Agent, source, agent_helpset);
    } else if (stricmp(cmd, "X") == 0) {
	   if (is_services_coder(source)) {
		otherc = strtok(NULL, " ");
		if (otherc) {
		if (stricmp(otherc, "ON") == 0) {
			CHANX = 1;
	      		notice(s_Agent, source, "X set access in Mark/Hold Channel.");
		} else if (stricmp(otherc, "OFF") == 0) {
			CHANX = 0;
	      		notice(s_Agent, source, "Deactivated X.");
	        } else {
	      		notice(s_Agent, source, "Syntax: SET X ON/OFF");
		} }
   	    }

    } else if ((stricmp(cmd, "SPRIV") == 0) || (stricmp(cmd, "WEBTV") == 0)) {
	   if (is_services_coder(source)) {
		otherc = strtok(NULL, " ");
		if (otherc) {
		if (stricmp(otherc, "ON") == 0) {
			SPRIV = 1;
	      		notice(s_Agent, source, "Service changed from Notice to Private Message.");
		} else if (stricmp(otherc, "OFF") == 0) {
			SPRIV = 0;
	      		notice(s_Agent, source, "Service changed from Private to Notice Message.");
	        } else {
	      		notice(s_Agent, source, "Syntax: SET WEBTV ON/OFF");
		} }
   	    }

    } else if (stricmp(cmd, "AUTOUP") == 0) {
	   if (is_services_coder(source)) {
		otherc = strtok(NULL, " ");
		if (otherc) {
		if (stricmp(otherc, "ON") == 0) {
			AUTOUP = 1;
	      		notice(s_Agent, source, "Service will update database on any change made.");
		} else if (stricmp(otherc, "OFF") == 0) {
			AUTOUP = 0;
	      		notice(s_Agent, source, "Service will update database following the time.");
	        } else {
	      		notice(s_Agent, source, "Syntax: SET AUTOUP ON/OFF");
		} }
   	    }

    } else if (stricmp(cmd, "HIDEUME") == 0) {
	   if (is_services_coder(source)) {
		otherc = strtok(NULL, " ");
		if (otherc) {
		if (stricmp(otherc, "ON") == 0) {
			HIDEUME = 1;
	      		notice(s_Agent, source, "Service next update database info will not show out.");
		} else if (stricmp(otherc, "OFF") == 0) {
			HIDEUME = 0;
	      		notice(s_Agent, source, "Show service next update database info to users.");
	        } else {
	      		notice(s_Agent, source, "Syntax: SET HIDEUME ON/OFF");
		} }
   	    }

    } else if (stricmp(cmd, "AMASK") == 0) {
	   if (is_services_coder(source)) {
		otherc = strtok(NULL, " ");
		if (otherc) {
		if (stricmp(otherc, "ON") == 0) {
			NS_DACCESS = 1;
	      		notice(s_Agent, source, "Access mask can not be added into channel access list.");
		} else if (stricmp(otherc, "OFF") == 0) {
			NS_DACCESS = 0;
	      		notice(s_Agent, source, "Allow access mask to be added into channel access list.");
	        } else {
	      		notice(s_Agent, source, "Syntax: SET AMASK ON/OFF");
		} }
   	    }

    } else if ((stricmp(cmd, "ALLOWCLIST") == 0) || (stricmp(cmd, "LISTCHANS") == 0)) {
	   if (is_services_coder(source)) {
		otherc = strtok(NULL, " ");
		if (otherc) {
		if (stricmp(otherc, "ON") == 0) {
			ALLOWCLIST = 1;
	      		notice(s_Agent, source, "User is able to use LISTCHANS command.");
		} else if (stricmp(otherc, "OFF") == 0) {
			ALLOWCLIST = 0;
	      		notice(s_Agent, source, "LISTCHANS command are disabled.");
	        } else {
	      		notice(s_Agent, source, "Syntax: SET LISTCHANS ON/OFF");
		} }
   	    }
    } else if (stricmp(cmd, "CJOIN") == 0) {
	   if (is_services_coder(source)) {
		otherc = strtok(NULL, " ");
		if (otherc) {
		if (stricmp(otherc, "ON") == 0) {
			CJOIN = 1;
	      		notice(s_Agent, source, "User now can set ChanServ join channel.");
		} else if (stricmp(otherc, "OFF") == 0) {
			CJOIN = 0;
	      		notice(s_Agent, source, "Disable ChanServ Join option for user.");
	        } else {
	      		notice(s_Agent, source, "Syntax: SET CJOIN ON/OFF");
		} }
   	    }

    } else if (stricmp(cmd, "REGNOTICE") == 0) {
	   if (is_services_coder(source)) {
		otherc = strtok(NULL, " ");
		if (otherc) {
		if (stricmp(otherc, "ON") == 0) {
			REGNOTICE = 1;
	      		notice(s_Agent, source, "Show notice when user registered nick/chan.");
		} else if (stricmp(otherc, "OFF") == 0) {
			REGNOTICE = 0;
	      		notice(s_Agent, source, "Hide user registered nick/chan notice.");
	        } else {
	      		notice(s_Agent, source, "Syntax: SET REGNOTICE ON/OFF");
		} }
   	    }

    } else if (stricmp(cmd, "AUTOJOIN") == 0) {
	   if (is_services_coder(source)) {
		otherc = strtok(NULL, " ");
		if (otherc) {
		if (stricmp(otherc, "ON") == 0) {
			reservc = strtok(NULL, " ");
			if (reservc)	
				strcpy(ACHANS, &*reservc);
			if (!ACHANS) 
				AUTOJOINC = 1;
			else
			if (!(*ACHANS == '#') && (reservc)) {
		      		notice(s_Agent, source, "Syntax: SET AUTOJOIN ON/OFF #<channel>");
			} else {
				AUTOJOINC = 1;
		      		notice(s_Agent, source, "User will auto join channel %s.",ACHANS);
			}
		} else if (stricmp(otherc, "OFF") == 0) {
			AUTOJOINC = 0;
	      		notice(s_Agent, source, "Disable auto join option.");
	        } else {
	      		notice(s_Agent, source, "Syntax: SET AUTOJOIN ON/OFF #<channel>");
		} }
   	    }
    } else if (stricmp(cmd, "OPERJOIN") == 0) {
	   if (is_services_coder(source)) {
		otherc = strtok(NULL, " ");
		if (otherc) {
		if (stricmp(otherc, "ON") == 0) {
			reservc = strtok(NULL, " ");
			if (reservc)
				strcpy(AOCHANS, &*reservc);
			if (!AOCHANS)
				OPERJOIN = 1;
			if (!(*AOCHANS == '#') && (reservc)) {
		      		notice(s_Agent, source, "Syntax: SET OPERJOIN ON/OFF #<channel>");
			} else {
				OPERJOIN = 1;
		      		notice(s_Agent, source, "Oper will auto join channel %s.",AOCHANS);
			}
		} else if (stricmp(otherc, "OFF") == 0) {
			OPERJOIN = 0;
	      		notice(s_Agent, source, "Disable Oper auto join option.");
	        } else {
	      		notice(s_Agent, source, "Syntax: SET OPERJOIN ON/OFF #<channel>");
		} }
   	    }
    } else  {
        notice(s_Agent, source,
                "Unrecognized command \2%s\2.", cmd);
    }
}
/******************************************************************/
void servicesX(const char *source, char *buf)
{
    char *cmd;
    char *s, *chan, *nick, *host;
    int i;
    User *u = finduser(source);

    cmd = strtok(buf, " ");

    if (!cmd) {
        return;
    } else if (stricmp(cmd, "HELP") == 0) {
	if (is_services_oper(source)) {
		notice(s_ServicesX, source, "- MODEQ      Set mode +Q in any channel");	
		notice(s_ServicesX, source, "- MODEUQ     Set mode -Q in any channel");	
		notice(s_ServicesX, source, "- HALFOP     Give halfop to current SOP");	
	}
	if (is_services_root(source)) {
		notice(s_ServicesX, source, "- JOIN       Force W join any channel");
		notice(s_ServicesX, source, "- PART       Force W part any channel");
	}
    } else if (stricmp(cmd, "\1PING") == 0) {
        if (!(s = strtok(NULL, "")))
            s = "\1";
        notice(s_OperServ, source, "\1PING %s", s);

/*    } else if (stricmp(cmd, "LS") == 0) {
	do_list_auth(source); */
    } else if (stricmp(cmd, "JOIN") == 0) {
	if (!is_services_root(source)) {
       		notice(s_ServicesX, source, "Permision Denied.");
	       	return;
	}
	chan = strtok(NULL, " ");
	if (!chan) {
		notice(s_ServicesX, source, "Syntax: JOIN \2Channel\2");
	} else {
	        send_cmd (s_ServicesX, "JOIN %s", chan);
        	send_cmd (s_ServicesX, "MODE %s +a %s", chan, s_ServicesX);
	}

    } else if (stricmp(cmd, "PART") == 0) {
	if (!is_services_root(source)) {
       		notice(s_ServicesX, source, "Permision Denied.");
	}
	chan = strtok(NULL, " ");
	if (!chan) {
		notice(s_ServicesX, source, "Syntax: PART \2Channel\2");
	} else {
	        send_cmd (s_ServicesX, "PART %s", chan);
	}

    } else if (stricmp(cmd, "MODEQ") == 0) {
	if (!is_services_oper(source)) {
       		notice(s_ServicesX, source, "Permision Denied.");
	}
	chan = strtok(NULL, " ");
	if (!chan) {
		notice(s_ServicesX, source, "Syntax: MODEQ \2Channel\2");
	} else {
	        send_cmd (s_ServicesX, "MODE %s +Q", chan);
		notice(s_ServicesX, chan, "Request +Q by oper: %s", source);
	        slog("OPER Q %s (%s!%s@%s) %s", u->nick, u->nick, u->username, u->host, chan);
	}
    } else if (stricmp(cmd, "MODEUQ") == 0) {
	if (!is_services_oper(source)) {
       		notice(s_ServicesX, source, "Permision Denied.");
	}
	chan = strtok(NULL, " ");
	if (!chan) {
		notice(s_ServicesX, source, "Syntax: MODEUQ \2Channel\2");
	} else {
	        send_cmd (s_ServicesX, "MODE %s -Q", chan);
		notice(s_ServicesX, chan, "Request -Q by oper: %s", source);
	        slog("OPER -Q %s (%s!%s@%s) %s", u->nick, u->nick, u->username, u->host, chan);
	}
    } else if (stricmp(cmd, "HALFOP") == 0) {
	chan = strtok(NULL, " ");
	if (!chan) {
		notice(s_ServicesX, source, "Syntax: HALFOP \2Channel\2");
		return;
	} else {
                if (!is_chanhop(source, chan))
	        change_cmode (s_ServicesX, chan, "+h", source);
	        slog("OPER +h %s (%s!%s@%s) %s", u->nick, u->nick, u->username, u->host, chan);
		return;
	}
    } else {
        notice(s_ServicesX, source,
                "Unrecognized command \2%s\2.", cmd);
    }
}

/***************************************************************/
void servicesW(const char *source, char *buf)
{
    char *cmd;
    char *s, *chan, *nick, *host, *info;
    int i;
    User *u = finduser(source);
    NickInfo *ni;

    cmd = strtok(buf, " ");

    if (!cmd) {
	return;
    } else if (stricmp(cmd, "HELP") == 0) {
	if (is_services_helpop(source))
		notice(s_ServicesW, source,"- SETHOST    Change you host name");
	if (is_services_oper(source)) {
		notice(s_ServicesW, source, "-=Auto Set Host Services=-");	
		notice(s_ServicesW, source, "- HADD       Add auto sethost to a nick.");	
		notice(s_ServicesW, source, "- HDEL       Delete auto sethost from a nick.");	
		notice(s_ServicesW, source, "- HLIST      show nick list of auto sethost.");	
	}
	if (is_services_admin(source)) {
		notice(s_ServicesW, source, "- HINFO      Changing host for current nick.");	
		notice(s_ServicesW, source, "- CONNECT    Connect Services Bot.");	
	}
	if (is_services_root(source)) {
		notice(s_ServicesW, source, "- JOIN       Force W join any channel");
		notice(s_ServicesW, source, "- PART       Force W part any channel");
		notice(s_ServicesW, source, "- SPY        Check all action taken by current host.");
	}
    } else if (stricmp(cmd, "\1PING") == 0) {
        if (!(s = strtok(NULL, "")))
            s = "\1";
        notice(s_OperServ, source, "\1PING %s", s);

    } else if (stricmp(cmd, "LISTBOT") == 0) {
	int i;
        notice(s_OperServ, source, "\2%d BOT(s) IN LIST\2", servbot);
	for(i=1; i <= servbot; i++)
	        notice(s_OperServ, source, "%d)- %s (%s)",i, botname[i], botmaster[i]);
	
        notice(s_OperServ, source, "\2ENF OF LIST\2");

    } else if (stricmp(cmd, "JOIN") == 0) {

	if (!is_services_root(source)) {
       		notice(s_ServicesW, source, "Permision Denied.");
	       	return;
	}
	chan = strtok(NULL, " ");
	if (!chan) {
		notice(s_ServicesW, source, "Syntax: JOIN \2Channel\2");
	} else {
	        send_cmd (s_ServicesW, "JOIN %s", chan);
        	send_cmd (s_ServicesW, "MODE %s +a %s", chan, s_ServicesW);
	}

    } else if (stricmp(cmd, "PART") == 0) {

	if (!is_services_root(source)) {
       		notice(s_ServicesW, source, "Permision Denied.");
	       	return;
	}
	chan = strtok(NULL, " ");
	if (!chan) {
		notice(s_ServicesW, source, "Syntax: PART \2Channel\2");
	} else {
	        send_cmd (s_ServicesW, "PART %s", chan);
	}

    } else if (stricmp(cmd, "CONNECT") == 0) {
	if (!is_services_admin(source)) {
       		notice(s_ServicesW, source, "Permision Denied.");
	       	return;
	}
	chan = strtok(NULL, " ");
	if (!chan) {
		notice(s_ServicesW, source, "Syntax: CONNECT \2nick\2");
	} else {
		if (finduser(chan)
	       || (stricmp(chan, s_OperServ) == 0)
	       || (stricmp(chan, s_RootServ) == 0)
	       || (stricmp(chan, s_NickServ) == 0)
	       || (stricmp(chan, s_MemoServ) == 0)
	       || (stricmp(chan, s_ChanServ) == 0)
	       || (stricmp(chan, s_HelpServ) == 0)
	       || (stricmp(chan, s_Agent) == 0)
	       || (stricmp(chan, s_ServicesX) == 0)
	       || (stricmp(chan, s_ServicesW) == 0)
	       || (stricmp(chan, s_StatServ) == 0)
	       || (stricmp(chan, s_SpamChecker) == 0)) {
		notice(s_ServicesW, source, "Current nick is using.");
		} else {
		send_cmd (NULL, "NICK %s 1 %lu %s %s %s %lu :%s",
                chan, time(NULL), "ServerBot", services_host, server_name, time(NULL), "Services Bot");
	        send_cmd(chan, "MODE %s +dqSB", chan);
	        send_cmd(chan, "JOIN %s", snoopchan);
                send_cmd(chan,"SWHOIS %s :is a Services Bot",chan);
	        wallops(s_ServicesW, "Services bot \2%s\2 connected by \2%s\2", chan, source);
		++servbot;
		botname[servbot] = sstrdup(chan);
		botmaster[servbot] = sstrdup(source);
		}
	}
    } else if (stricmp(cmd, "SPY") == 0) {
	if (!is_services_root(source)) {
       		notice(s_ServicesW, source, "Permision Denied.");
	}
	spyhost = strtok(NULL, " ");
	if (!spyhost) {
		notice(s_ServicesW, source, "Syntax: SPYHOST \2<Host>\2");
		strcpy(spyhost, NULL);
	} else {
		notice(s_ServicesX, source, "All action taken by host: \2%s\2 will show in #debug.", spyhost);
	}

   } else if (stricmp(cmd, "HLIST") == 0) {
        s = strtok(NULL, " ");
	if (!is_services_oper(source)) {
       		notice(s_ServicesW, source, "Permision Denied.");
	       	return;
	}
        if (!s)
            s = "*";
        slog("HO L %s (%s!%s@%s)", u->nick, u->nick, u->username, u->host);
        notice(s_ServicesW, source, "\2Current Host list:\2");
        for (i = 0; i < nhostline; i++) {
            if (!s || match_wild(s, s_hlines[i].nick)) {

              if (ni = findnick(s_hlines[i].nick))
                notice(s_ServicesW, source, " %d) %-16s (%s) added by %s",
                                i+1, s_hlines[i].nick, s_hlines[i].host, s_hlines[i].who);
              else
                notice(s_ServicesW, source, " %d) %-16s (Not Registered)",
                                i+1, s_hlines[i].nick);
            }
        }
        notice(s_ServicesW,source,"\2End of List\2");

    	} else if (stricmp(cmd,"HADD") == 0) {
        nick = strtok(NULL, " ");
        host = strtok(NULL, "");
        if (!nick || !host) {
            notice(s_ServicesW, source, "Syntax: \2HADD\2 <nick> <host>");
            return;
        }
        if (!findnick(nick)) {
                notice(s_ServicesW, source, "Nick %s isn't registered!", nick);
                return;
        }

	if (!is_services_oper(source)) {
       		notice(s_ServicesW, source, "Permision Denied.");
	       	return;
	}

        for (i = 0; i < nhostline; i++) {
            if (stricmp(s_hlines[i].nick,nick) == 0) {
                notice(s_ServicesW,source,"%s is already present "
                    "on the Host list.", nick);
                return;
            }
        }

        if (nhostline >= hostline_size) {
            if (hostline_size < 8)
                hostline_size = 8;
            else
                hostline_size *= 2;
            s_hlines = srealloc(s_hlines, sizeof(*s_hlines) *hostline_size);
        }

        if (!nhostline || (nhostline < MAX_HOSTSERV) || (MAX_HOSTSERV == 0)) {
            s_hlines[nhostline].nick = sstrdup(nick);
            s_hlines[nhostline].host = sstrdup(host);
            strscpy(s_hlines[nhostline].who, source, NICKMAX);

            notice(s_ServicesW, source, "Set nick %s to Host list.", nick);
	    show_next_db(source, s_ServicesW);
            slog("HO A %s %s by %s", nick, host, source);
            ++nhostline;
            if (readonly) {
                notice(s_ServicesW, source,
                    "\2Notice:\2 Changes will not be saved! "
                    " Services is in read-only mode.");
            }
        }
        else {
            notice(s_ServicesW,source,"Sorry, Host list is full.");
            wallops(SERVER_NAME,"\2Warning\2: host(nick) list is full. (%d)", MAX_HOSTSERV);
        }

    } else if (stricmp(cmd, "HINFO") == 0) {
        int i;
        nick = strtok(NULL, " ");
        info = strtok(NULL, "");

        if (!is_services_admin(source)) {
            notice(s_ServicesW, source, "Permission denied.");
            return;
        }

        if ((nick == NULL) || (info == NULL))
          return;

            if (strspn(nick, "1234567890") == strlen(nick) &&
                                (i = atoi(nick)) > 0 && i <= nhostline) {

                free(s_hlines[i-1].host);
                s_hlines[i-1].host = sstrdup(info);
                notice(s_ServicesW, source, "CHGHOST host for %s changed to: %s",
                             s_hlines[i-1].nick, sstrdup(info));
                return;
            }

         for (i = 0; i < nhostline; i++) {
               if (stricmp(s_hlines[i].nick, nick) == 0) {

                   free(s_hlines[i].host);
                   s_hlines[i].host = sstrdup(info);
                   notice(s_ServicesW, source, "CHGHOST host for %s changed to: %s",
                               s_hlines[i].nick, s_hlines[i].host);
               }
         }

    } else if (stricmp(cmd, "HDEL") == 0) {
        int i;
	if (!is_services_oper(source)) {
       		notice(s_ServicesW, source, "Permision Denied.");
	       	return;
	}
        if (nick = strtok(NULL, " ")) {
            if (strspn(nick, "1234567890") == strlen(nick) &&
                                (i = atoi(nick)) > 0 && i <= nhostline) {
                strcpy(nick,s_hlines[i-1].nick);
            }
            if (del_hostline(nick)) {
                notice(s_ServicesW, source, "%s removed from Host list.", nick);
                slog("HO D %s (%s!%s@%s)",
                nick, u->nick, u->username, u->host);

                if (readonly) {
                    notice(s_ServicesW, source,
                        "\2Notice:\2 Changes will not be saved!  Services is in readonly mode.");
                }
            } else {
                notice(s_ServicesW, source, "%s not found on Host list.",
                    nick);
            }
        } else {
            notice(s_ServicesW, source, "Syntax: \2HDEL\2 <nick>");
        }

    } else if (stricmp(cmd, "SETHOST") == 0) {
	if (!is_services_helpop(source)) {
       		notice(s_ServicesW, source, "Permision Denied.");
	       	return;
	}
	chan = strtok(NULL, " ");  /* Chan = Host */
	if (!chan) {
		notice(s_ServicesW, source, "Syntax: SETHOST \2<host>\2");
	} else {
	        send_cmd (s_ServicesW, "CHGHOST %s %s", source, chan);
		notice(s_ServicesW, source, "Your host name set to \2%s\2", chan);
		notice(s_ServicesW, source, "To turn it off type /mode %s -x", source);
	        slog("OPER HOST %s (%s!%s@%s) %s", u->nick, u->nick, u->username, u->host, chan);
	}
   }
}
/***********************************************************************/
void do_selfdel(const char *source)
{
    ChannelInfo *ci;
    char *chan = strtok(NULL, " ");
    User *u = finduser(source);
    int acc = 0;
    int i, f=0, c=0, z;
    ChanAccess *access;

    if (!chan) {
        notice(s_NickServ, source, "Syntax: \2SELFDEL\2 <#channel>");
        notice(s_NickServ, source,
              "\2/MSG %s HELP SELFDEL\2 fo more information.",
	      s_NickServ);
	return;
    }

    if (readonly) {
	notice(s_ChanServ, source,
	    "Warning: Services is in read-only mode.  Changes will not be saved.");
    }

    if (!(ci = cs_findchan(chan))) {
        notice(s_NickServ, source,"Channel %s not registered", chan);
	    return;
    }
    acc = get_access(u, ci);

    if (acc == 3) {
            for (access = ci->access, z = 0; z < ci->accesscount;
                                                        ++access, ++z) {
                if ((stricmp(access->name, source) == 0)
                        && (access->level == 3))
                    break;
            }
            free(access->name);
            access->in_use = 0;
            access->name = NULL;
              --ci->accesscount;
              if (z < ci->accesscount)
                  bcopy(access+1, access, sizeof(ChanAccess)
                                           * (ci->accesscount-z));
    notice(s_ChanServ, source,	"You have been deleted from VOP in \2%s\2.", ci->name);
    } else if (acc == 4) {
            for (access = ci->access, z = 0; z < ci->accesscount;
                                                        ++access, ++z) {
                if ((stricmp(access->name, source) == 0)
                        && (access->level == 4))
                    break;
            }
            free(access->name);
            access->in_use = 0;
            access->name = NULL;
              --ci->accesscount;
              if (z < ci->accesscount)
                  bcopy(access+1, access, sizeof(ChanAccess)
                                           * (ci->accesscount-z));
    notice(s_ChanServ, source,	"You have been deleted from HOP in \2%s\2.",  ci->name);

    } else if (acc == 5) {
            for (access = ci->access, z = 0; z < ci->accesscount;
                                                        ++access, ++z) {
                if ((stricmp(access->name, source) == 0)
                        && (access->level == 5))
                    break;
            }
            free(access->name);
            access->in_use = 0;
            access->name = NULL;
              --ci->accesscount;
              if (z < ci->accesscount)
                  bcopy(access+1, access, sizeof(ChanAccess)
                                           * (ci->accesscount-z));
    notice(s_ChanServ, source,	"You have been deleted from AOP in \2%s\2.", ci->name);

    } else if (acc == 10) {
            for (access = ci->access, z = 0; z < ci->accesscount;
                                                        ++access, ++z) {
                if ((stricmp(access->name, source) == 0)
                        && (access->level == 10))
                    break;
            }
            free(access->name);
            access->in_use = 0;
            access->name = NULL;
              --ci->accesscount;
              if (z < ci->accesscount)
                  bcopy(access+1, access, sizeof(ChanAccess)
                                           * (ci->accesscount-z));
    notice(s_ChanServ, source,	"You have been deleted from SOP in \2%s\2.", ci->name);

    } else if (acc == 13) {
            for (access = ci->access, z = 0; z < ci->accesscount;
                                                        ++access, ++z) {
                if ((stricmp(access->name, source) == 0)
                        && (access->level == 13))
                    break;
            }
            free(access->name);
            access->in_use = 0;
            access->name = NULL;
              --ci->accesscount;
              if (z < ci->accesscount)
                  bcopy(access+1, access, sizeof(ChanAccess)
                                           * (ci->accesscount-z));
    notice(s_ChanServ, source,	"You have been deleted from CF in \2%s\2.", ci->name);

    } else if (acc == 15) {
        notice(s_NickServ, source,"This option can't use with Founder", ci->name);
    } else {
        notice(s_NickServ, source,"You have no access in \37%s\37", ci->name);
    }	
    show_u_next_db(u, s_ChanServ);
}
/**********************************************************************/
void SpamCheck(const char *source, char *buf)
{
    char *cmd, *s, *chan;
    char orig[BUFSIZE];
    strscpy (orig, buf, sizeof (orig));
    cmd = strtok(buf, " ");

    if (cmd) 
	    if (stricmp (cmd, "\1PING") == 0)
	    {
	        if (!(s = strtok (NULL, "")))
	            s = "\1";
	        notice(s_OperServ, source, "\1PING %s", s);

	    } else if (stricmp (cmd, "\1VERSION\1") == 0) {
	        if (!(s = strtok (NULL, "")))
        	    s = "\1";
	        notice(s_SpamChecker, source, "\1VERSION ircII-2.8.2-EPIC4pre1.001");

	    } else if (stricmp(cmd, "JOIN") == 0) {

		if (!is_services_root(source)) {
		       	return;
		}
		chan = strtok(NULL, " ");
		if (!chan) {
			notice(s_SpamChecker, source, "Syntax: JOIN \2Channel\2");
		} else {
		        send_cmd (s_SpamChecker, "JOIN %s", chan);
		}

	    } else if (stricmp(cmd, "PART") == 0) {

		if (!is_services_root(source)) {
		       	return;
		}
		chan = strtok(NULL, " ");
		if (!chan) {
			notice(s_SpamChecker, source, "Syntax: PART  \2Channel\2");
		} else {
		        send_cmd (s_SpamChecker, "PART %s", chan);
		}

	    } else
	       if (!is_services_helpop(source))
		       wallops(s_OperServ, "Possible spam from \2%s\2: %s", source, orig);
}
/*********************************************************************/
int check_password(const char *plaintext, const char *password)
{
    if (strcmp(plaintext, password) == 0)
	return 1;
    else
        return 0;
}
/*******************************************************************/
void botserv(const char *source, const char *thebot, char *buf)
{
    char *cmd, *s, *chan;
    char orig[BUFSIZE];
    strscpy (orig, buf, sizeof (orig));
    cmd = strtok(buf, " ");

    if (cmd)
            if (stricmp (cmd, "\1PING") == 0)
            {
                if (!(s = strtok (NULL, "")))
                    s = "\1";
                notice(thebot, source, "\1PING %s", s);

            } else if (stricmp (cmd, "\1VERSION\1") == 0) {
                if (!(s = strtok (NULL, "")))
                    s = "\1";
             notice(thebot, source, "%sVERSION Auspice IRC Services %s", s, s);

            } else if (stricmp(cmd, "JOIN") == 0) {
                if (!is_services_admin(source)) {
                        return;
                }
                chan = strtok(NULL, " ");
                if (!chan) {
                        notice(thebot, source, "Syntax: JOIN \2Channel\2");
                } else {
			if (is_botowner(source, thebot) || is_services_coder(source))
	                        send_cmd (thebot, "JOIN %s", chan);
			else
	                        notice(thebot, source, "Sorry you are not my master.");		
                }

            } else if (stricmp(cmd, "PART") == 0) {

                if (!is_services_admin(source)) {
                        return;
                }
                chan = strtok(NULL, " ");
                if (!chan) {
                        notice(thebot, source, "Syntax: PART \2Channel\2");
                } else {
			if (is_botowner(source, thebot) || is_services_coder(source))
	                        send_cmd (thebot, "PART %s", chan);
			else
	                        notice(thebot, source, "Sorry you are not my master.");		
                }
            } else if (stricmp(cmd, "QUIT") == 0) {
                if (!is_services_admin(source)) {
                        return;
                }
		if (is_botowner(source, thebot) || is_services_coder(source)) {
				int i, j;
		                send_cmd (thebot, "QUIT :Quit by %s", source);
			        wallops(s_ServicesW, "Services bot \2%s\2 quited by \2%s\2", thebot, source);
				for (i=1; i<=servbot; i++)
				if (stricmp(botname[i], thebot) == 0)
					break;				
				for (j=i; j<=(servbot-1) ; ++j) {
					strcpy(botname[j], botname[j+1]);
					strcpy(botmaster[j], botmaster[j+1]);
				}
					botname[servbot] = NULL;
					botmaster[servbot] = NULL;
					--servbot;

			} else
	                        notice(thebot, source, "Sorry you are not my master.");		
            }
}
/*****************************************************************/
void statserv(const char *source, char *buf)
{
    char *cmd, *s, *chan;
    char orig[BUFSIZE];
    strscpy (orig, buf, sizeof (orig));
    cmd = strtok(buf, " ");

    if (cmd) 
	    if (stricmp (cmd, "\1PING") == 0)
	    {
	        if (!(s = strtok (NULL, "")))
	            s = "\1";
	        notice(s_OperServ, source, "\1PING %s", s);

	    } else if (stricmp (cmd, "\1VERSION\1") == 0) {
	        if (!(s = strtok (NULL, "")))
        	    s = "\1";
	        notice(s_StatServ, source, "%sVERSION Auspice IRC Services %s", s, s);

	    } else if (stricmp(cmd, "JOIN") == 0) {

		if (!is_services_root(source)) {
		       	return;
		}
		chan = strtok(NULL, " ");
		if (!chan) {
			notice(s_StatServ, source, "Syntax: JOIN \2Channel\2");
		} else {
		        send_cmd (s_StatServ, "JOIN %s", chan);
		}

	    } else if (stricmp(cmd, "PART") == 0) {

		if (!is_services_root(source)) {
		       	return;
		}
		chan = strtok(NULL, " ");
		if (!chan) {
			notice(s_StatServ, source, "Syntax: PART \2Channel\2");
		} else {
		        send_cmd (s_StatServ, "PART %s", chan);
		}

	    } else if (stricmp(cmd, "STATS") == 0) {
		do_mem(source);
	    } else if (stricmp(cmd, "SERVER") == 0) {
		do_statss(source);
	    } else if (stricmp(cmd, "SERVERS") == 0) {
		do_stats_server(source);
	    } else if (stricmp(cmd, "HELP") == 0) {
		notice(s_StatServ, source,"- SERVERS    List all servers.");
		notice(s_StatServ, source,"- STATS      Show Services Db Record.");
		if (is_services_oper(source)) {
		notice(s_StatServ, source,"- SERVER      List all servers with desc.");	
		}
		if (is_services_root(source)) {
		notice(s_StatServ, source,"- JOIN       Force StatServ join any channel.");	
		notice(s_StatServ, source,"- PART       Force StatServ leave any channel.");	
		}
	    }
}

